"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaListener = exports.SchemaContextKindEnum = void 0;
const utils_1 = require("../utils");
var SchemaContextKindEnum;
(function (SchemaContextKindEnum) {
    SchemaContextKindEnum["VIEW"] = "view";
    SchemaContextKindEnum["CATALOG"] = "catalog";
    SchemaContextKindEnum["DATABASE"] = "database";
    SchemaContextKindEnum["TABLE"] = "table";
    SchemaContextKindEnum["COLUMN"] = "column";
})(SchemaContextKindEnum = exports.SchemaContextKindEnum || (exports.SchemaContextKindEnum = {}));
class SchemaListener {
    constructor() {
        this._schemaContexts = [];
    }
    getSchemaContexts() {
        return this._schemaContexts;
    }
    enterViewDef(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.VIEW,
            text: ctx.text,
            catalogName: ctx.databaseDef()?.catalogDef()?.catalogNameDef()
                ? (0, utils_1.sanitizeText)(ctx.databaseDef().catalogDef().catalogNameDef().text)
                : undefined,
            databaseName: ctx.databaseDef()?.databaseNameDef()
                ? (0, utils_1.sanitizeText)(ctx.databaseDef().databaseNameDef().text)
                : undefined,
            viewName: ctx.viewNameDef().text,
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
    enterCatalogDef(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.CATALOG,
            text: ctx.text,
            catalogName: (0, utils_1.sanitizeText)(ctx.catalogNameDef().text),
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
    enterDatabaseDef(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.DATABASE,
            text: ctx.text,
            catalogName: ctx.catalogDef()?.catalogNameDef()
                ? (0, utils_1.sanitizeText)(ctx.catalogDef().catalogNameDef().text)
                : undefined,
            databaseName: (0, utils_1.sanitizeText)(ctx.databaseNameDef().text),
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
    enterTableDef(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.TABLE,
            text: ctx.text,
            catalogName: ctx.databaseDef()?.catalogDef()?.catalogNameDef()
                ? (0, utils_1.sanitizeText)(ctx.databaseDef().catalogDef().catalogNameDef().text)
                : undefined,
            databaseName: ctx.databaseDef()?.databaseNameDef()
                ? (0, utils_1.sanitizeText)(ctx.databaseDef().databaseNameDef().text)
                : undefined,
            tableName: (0, utils_1.sanitizeText)(ctx.tableNameDef().text),
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
    enterTable(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.TABLE,
            text: ctx.tableDef()?.tableNameDef()?.text || '',
            catalogName: ctx.tableDef()?.databaseDef()?.catalogDef()?.catalogNameDef()
                ? (0, utils_1.sanitizeText)(ctx.tableDef().databaseDef().catalogDef().catalogNameDef().text)
                : undefined,
            databaseName: ctx.tableDef()?.databaseDef()?.databaseNameDef()
                ? (0, utils_1.sanitizeText)(ctx.tableDef().databaseDef().databaseNameDef().text)
                : undefined,
            tableName: (0, utils_1.sanitizeText)(ctx.tableDef()?.tableNameDef()?.text || ''),
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
    enterCreateTable(ctx) {
        ctx
            .createOrReplaceTableColTypeList()
            ?.createOrReplaceTableColType()
            ?.forEach(col => {
            this._schemaContexts.push({
                kind: SchemaContextKindEnum.COLUMN,
                text: col._colName.text,
                columnName: col._colName.text || '',
                catalogName: (0, utils_1.sanitizeText)(ctx.createTableHeader().tableDef()?.databaseDef()?.catalogDef()?.catalogNameDef()?.text || ''),
                databaseName: (0, utils_1.sanitizeText)(ctx.createTableHeader().tableDef()?.databaseDef()?.databaseNameDef()?.text || ''),
                tableName: (0, utils_1.sanitizeText)(ctx.createTableHeader().tableDef()?.tableNameDef()?.text || ''),
                startOffset: col.start.startIndex,
                endOffset: col.start.startIndex + col._colName.text.length
            });
        });
    }
    enterColumnDef(ctx) {
        this._schemaContexts.push({
            kind: SchemaContextKindEnum.COLUMN,
            text: ctx.columnNameDef()?.text || '',
            catalogName: ctx?.tableDef()?.databaseDef()?.catalogDef()?.catalogNameDef()
                ? (0, utils_1.sanitizeText)(ctx.tableDef().databaseDef().catalogDef().catalogNameDef().text)
                : undefined,
            databaseName: ctx?.tableDef()?.databaseDef()?.databaseNameDef()
                ? (0, utils_1.sanitizeText)(ctx.tableDef().databaseDef().databaseNameDef().text)
                : undefined,
            tableName: ctx?.tableDef()?.tableNameDef() ? ctx.tableDef().tableNameDef().text : undefined,
            columnName: ctx?.columnNameDef()?.text || '',
            startOffset: ctx.start.startIndex,
            endOffset: ctx.start.startIndex + ctx.text.length
        });
    }
}
exports.SchemaListener = SchemaListener;
//# sourceMappingURL=schema.listener.js.map