"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LSPServer = void 0;
const antlr4_c3_1 = require("antlr4-c3");
const tree_1 = require("antlr4ts/tree");
const sql_formatter_1 = require("sql-formatter");
const lsp = __importStar(require("vscode-languageserver"));
const vscode_languageserver_1 = require("vscode-languageserver");
const built_in_functions_1 = require("./assets/built-in-functions");
const SparkSqlLexer_1 = require("./lib/SparkSqlLexer");
const SparkSqlParser_1 = require("./lib/SparkSqlParser");
const schema_listener_1 = require("./listeners/schema.listener");
const statement_listener_1 = require("./listeners/statement.listener");
const parsing_warehouse_1 = require("./parsing-warehouse");
const protocol_translation_1 = require("./protocol-translation");
const schema_registry_1 = require("./schema-registry");
const utils_1 = require("./utils");
const ignoredTokens = [
    SparkSqlLexer_1.SparkSqlLexer.EOF,
    SparkSqlLexer_1.SparkSqlLexer.DOT,
    SparkSqlLexer_1.SparkSqlLexer.SEMI,
    SparkSqlLexer_1.SparkSqlLexer.COMMA,
    SparkSqlLexer_1.SparkSqlLexer.LEFT_BRACKET,
    SparkSqlLexer_1.SparkSqlLexer.RIGHT_BRACKET,
    SparkSqlLexer_1.SparkSqlLexer.EQ,
    SparkSqlLexer_1.SparkSqlLexer.DOUBLEQUOTED_STRING,
    SparkSqlLexer_1.SparkSqlLexer.BACKQUOTED_IDENTIFIER,
    SparkSqlLexer_1.SparkSqlLexer.AT,
    SparkSqlLexer_1.SparkSqlLexer.DIV,
    SparkSqlLexer_1.SparkSqlLexer.NSEQ,
    SparkSqlLexer_1.SparkSqlLexer.NEQ,
    SparkSqlLexer_1.SparkSqlLexer.NEQJ,
    SparkSqlLexer_1.SparkSqlLexer.LT,
    SparkSqlLexer_1.SparkSqlLexer.LTE,
    SparkSqlLexer_1.SparkSqlLexer.GT,
    SparkSqlLexer_1.SparkSqlLexer.GTE,
    SparkSqlLexer_1.SparkSqlLexer.PLUS,
    SparkSqlLexer_1.SparkSqlLexer.MINUS,
    SparkSqlLexer_1.SparkSqlLexer.IDENTIFIER
];
const preferredRules = [
    SparkSqlParser_1.SparkSqlParser.RULE_functionName,
    SparkSqlParser_1.SparkSqlParser.RULE_identifierReference,
    SparkSqlParser_1.SparkSqlParser.RULE_tableIdentifier,
    SparkSqlParser_1.SparkSqlParser.RULE_catalogNameDef,
    SparkSqlParser_1.SparkSqlParser.RULE_databaseNameDef,
    SparkSqlParser_1.SparkSqlParser.RULE_tableNameDef,
    SparkSqlParser_1.SparkSqlParser.RULE_viewNameDef,
    SparkSqlParser_1.SparkSqlParser.RULE_columnNameDef,
    SparkSqlParser_1.SparkSqlParser.RULE_identifier
];
class LSPServer {
    constructor() {
        this._warehouse = new parsing_warehouse_1.ParsingWarehouse();
        this._schemaRegistry = new schema_registry_1.SchemaRegistry();
    }
    get warehouse() {
        return this._warehouse;
    }
    get schemaRegistry() {
        return this._schemaRegistry;
    }
    doValidation(document) {
        const parsingValue = this._warehouse.getParsingValue(document);
        return parsingValue.errors;
    }
    doCompletion(document, position) {
        const parsingValue = this._warehouse.getParsingValue(document);
        let offset = document.offsetAt(position);
        const token = parsingValue.tokenStream.getTokens().find(t => t.startIndex <= offset && offset <= t.stopIndex + 1);
        if (!token) {
            return [];
        }
        const core = new antlr4_c3_1.CodeCompletionCore(parsingValue.parser);
        core.ignoredTokens = new Set(ignoredTokens);
        core.preferredRules = new Set(preferredRules);
        let index = token.tokenIndex;
        let bias = 0;
        if (token.type === SparkSqlLexer_1.SparkSqlLexer.DOT) {
            index += 1;
            offset -= 1;
            bias = 1;
        }
        const completionItems = [];
        try {
            const candidates = core.collectCandidates(index);
            const identifiers = [];
            if (candidates.rules.size > 0) {
                const listener = new schema_listener_1.SchemaListener();
                tree_1.ParseTreeWalker.DEFAULT.walk(listener, parsingValue.tree);
                const schemaContexts = listener.getSchemaContexts();
                const context = schemaContexts.find(c => c.startOffset <= offset && c.endOffset >= offset);
                const schemaParts = context?.text.replace(/[`'"]/g, '').split('.');
                if (candidates.rules.has(SparkSqlParser_1.SparkSqlParser.RULE_catalogNameDef)) {
                    const catalogsInDocument = (0, utils_1.unique)(schemaContexts.filter(c => c !== context && c.kind === schema_listener_1.SchemaContextKindEnum.CATALOG).map(c => c.catalogName));
                    completionItems.push(...catalogsInDocument.map(c => (0, protocol_translation_1.toCompletionItem)('catalog', c, `\`${c}\``, 'Catalog in document')), ...this._schemaRegistry
                        .getSchemaCatalogs()
                        .map(c => (0, protocol_translation_1.toCompletionItem)('catalog', c.label, `\`${c.label}\``, 'Schema catalog', c.description)));
                    identifiers.push('catalog');
                }
                if (candidates.rules.has(SparkSqlParser_1.SparkSqlParser.RULE_databaseNameDef)) {
                    const databasesInDocument = (0, utils_1.unique)(schemaContexts.filter(d => d !== context && d.kind === schema_listener_1.SchemaContextKindEnum.DATABASE).map(d => d.databaseName));
                    completionItems.push(...databasesInDocument.map(d => (0, protocol_translation_1.toCompletionItem)('database', d, d, 'Database in document')));
                    if (!!schemaParts && schemaParts.length <= 1 - bias) {
                        completionItems.push(...this._schemaRegistry
                            .getSchemaDatabases({})
                            .map(d => (0, protocol_translation_1.toCompletionItem)('database', d.label, d.label, `Schema database: ${d.catalog}.*`, d.description)));
                    }
                    else if (schemaParts) {
                        completionItems.push(...this._schemaRegistry
                            .getSchemaDatabases({ catalog: schemaParts?.[0] })
                            .map(d => (0, protocol_translation_1.toCompletionItem)('database', d.label, d.label, `Schema database: ${d.catalog}.*`, d.description)));
                    }
                    identifiers.push('database');
                }
                if (candidates.rules.has(SparkSqlParser_1.SparkSqlParser.RULE_tableNameDef)) {
                    let tablesInDocument = [];
                    if (!!schemaParts && schemaParts.length <= 1 - bias) {
                        completionItems.push(...this._schemaRegistry
                            .getSchemaTables({})
                            .map(t => (0, protocol_translation_1.toCompletionItem)('table', t.label, t.label, `Schema table: ${t.catalog}.${t.database}.*`, t.description)));
                        tablesInDocument = (0, utils_1.unique)(schemaContexts.filter(t => t !== context && t.kind === schema_listener_1.SchemaContextKindEnum.TABLE).map(t => t.tableName));
                    }
                    else if (!!schemaParts && schemaParts.length === 2 - bias) {
                        completionItems.push(...this._schemaRegistry
                            .getSchemaTables({ database: schemaParts[0] })
                            .map(t => (0, protocol_translation_1.toCompletionItem)('table', t.label, t.label, `Schema table: ${t.catalog}.${t.database}.*`, t.description)));
                        tablesInDocument = (0, utils_1.unique)(schemaContexts.filter(t => t !== context &&
                            t.kind === schema_listener_1.SchemaContextKindEnum.TABLE &&
                            (!schemaParts[0] || t.databaseName === schemaParts[0])).map(t => t.tableName));
                    }
                    else {
                        completionItems.push(...this._schemaRegistry
                            .getSchemaTables({ catalog: schemaParts?.[0], database: schemaParts?.[1] })
                            .map(t => (0, protocol_translation_1.toCompletionItem)('table', t.label, t.label, `Schema table: ${t.catalog}.${t.database}.*`, t.description)));
                        tablesInDocument = (0, utils_1.unique)(schemaContexts.filter(t => t !== context &&
                            t.kind === schema_listener_1.SchemaContextKindEnum.TABLE &&
                            (!schemaParts?.[0] || t.catalogName === schemaParts?.[0]) &&
                            (!schemaParts?.[1] || t.databaseName === schemaParts?.[1])).map(t => t.tableName));
                    }
                    completionItems.push(...tablesInDocument.map(t => (0, protocol_translation_1.toCompletionItem)('table', t, t, 'Table in document')));
                    identifiers.push('table');
                }
                if (candidates.rules.has(SparkSqlParser_1.SparkSqlParser.RULE_columnNameDef)) {
                    let nearestTable;
                    let columnDefinitionMap = undefined;
                    const [startIndex, endIndex] = this.getSemicolonRange(document.getText(), offset);
                    if (startIndex && endIndex) {
                        nearestTable = schemaContexts.find(c => c.kind === schema_listener_1.SchemaContextKindEnum.TABLE && c.startOffset >= startIndex && c.endOffset <= endIndex);
                    }
                    const { catalogName, databaseName, tableName } = nearestTable || {};
                    let columns;
                    if (!!schemaParts && schemaParts.length <= 1) {
                        if (bias > 0) {
                            columnDefinitionMap = {
                                table: schemaParts[0]
                            };
                        }
                        columns = this._schemaRegistry.getSchemaColumns({
                            catalog: catalogName,
                            database: databaseName,
                            table: bias > 0 ? schemaParts[0] : tableName
                        });
                    }
                    else if (!!schemaParts && schemaParts.length === 2) {
                        columnDefinitionMap = {
                            table: schemaParts[0]
                        };
                        columns = this._schemaRegistry.getSchemaColumns({ table: schemaParts[0] || tableName });
                    }
                    else if (!!schemaParts && schemaParts.length === 3) {
                        columnDefinitionMap = {
                            database: schemaParts[0],
                            table: schemaParts[1]
                        };
                        columns = this._schemaRegistry.getSchemaColumns({
                            database: schemaParts[0] || databaseName,
                            table: schemaParts[1] || tableName
                        });
                    }
                    else {
                        columnDefinitionMap = {
                            catalog: schemaParts?.[0],
                            database: schemaParts?.[1],
                            table: schemaParts?.[2]
                        };
                        columns = this._schemaRegistry.getSchemaColumns({
                            catalog: schemaParts?.[0] || catalogName,
                            database: schemaParts?.[1] || databaseName,
                            table: schemaParts?.[2] || tableName
                        });
                    }
                    const columnsInDocument = (0, utils_1.unique)(schemaContexts.filter(t => {
                        return (t !== context &&
                            t.kind === schema_listener_1.SchemaContextKindEnum.COLUMN &&
                            (columnDefinitionMap
                                ? (!columnDefinitionMap.catalog || columnDefinitionMap.catalog === t.catalogName) &&
                                    (!columnDefinitionMap.database || columnDefinitionMap.database === t.databaseName) &&
                                    (!columnDefinitionMap.table || columnDefinitionMap.table === t.tableName)
                                : (!t.tableName && !nearestTable) ||
                                    !nearestTable?.tableName ||
                                    t.tableName === nearestTable.tableName));
                    }));
                    completionItems.push(...columnsInDocument.map(d => (0, protocol_translation_1.toCompletionItem)('column', d.text, d.text, `Column ${d.tableName ? `of ${d.tableName}` : ''} in document`)));
                    if (columns.length) {
                        completionItems.push(...columns.map(c => (0, protocol_translation_1.toCompletionItem)('column', c.label, c.label, `Schema column: ${c.catalog}.${c.database}.${c.table}.*`, c.description)));
                    }
                    identifiers.push('column');
                }
                if (candidates.rules.has(SparkSqlParser_1.SparkSqlParser.RULE_functionName)) {
                    let functions;
                    if (!!schemaParts && schemaParts.length <= 1 - bias) {
                        functions = this._schemaRegistry.getSchemaFunctions({});
                    }
                    else if (!!schemaParts && schemaParts.length === 2 - bias) {
                        functions = this._schemaRegistry.getSchemaFunctions({ database: schemaParts[0] });
                    }
                    else {
                        functions = this._schemaRegistry.getSchemaFunctions({
                            catalog: schemaParts?.[0],
                            database: schemaParts?.[1]
                        });
                    }
                    completionItems.push(...functions.map(f => (0, protocol_translation_1.toCompletionItem)('function', f.label, f.label, `UDF: ${f.catalog}.${f.database}.*`, f.description)), ...built_in_functions_1.BuiltInFunctions.map(f => (0, protocol_translation_1.toCompletionItem)('built-in', f.name, f.name, 'built-in function', f.description)));
                    identifiers.push('function');
                }
            }
            for (const candidate of Array.from(candidates.tokens)) {
                const keyword = parsingValue.parser.vocabulary.getDisplayName(candidate[0]).replace(/'/g, '');
                completionItems.push({
                    kind: vscode_languageserver_1.CompletionItemKind.Keyword,
                    label: keyword,
                    insertText: keyword,
                    insertTextFormat: vscode_languageserver_1.InsertTextFormat.Snippet
                });
            }
        }
        catch (e) {
        }
        return completionItems;
    }
    doFormatting(document, _options) {
        const parsingValue = this._warehouse.getParsingValue(document);
        try {
            const formattedCode = (0, sql_formatter_1.formatDialect)(parsingValue.origin, {
                dialect: sql_formatter_1.spark,
                tabWidth: 2,
                useTabs: false,
                keywordCase: 'preserve',
                linesBetweenQueries: 1,
                newlineBeforeSemicolon: true
            });
            const textEdit = lsp.TextEdit.replace(lsp.Range.create(0, 0, document.lineCount, 0), formattedCode);
            return [textEdit];
        }
        catch (e) {
            return [];
        }
    }
    doFoldingRanges(document) {
        const parsingValue = this._warehouse.getParsingValue(document);
        const listener = new statement_listener_1.StatementListener(document);
        tree_1.ParseTreeWalker.DEFAULT.walk(listener, parsingValue.tree);
        const ranges = listener.getRanges();
        return ranges.map(range => (0, protocol_translation_1.toFoldingRange)(range));
    }
    getTokenByPosition(document, position) {
        const parsingValue = this._warehouse.getParsingValue(document);
        const offset = document.offsetAt(position);
        return parsingValue.tokenStream.getTokens().find(t => t.startIndex <= offset && offset <= t.stopIndex + 1);
    }
    getSemicolonRange(str, target) {
        if (target < 0 || target >= str.length) {
            return [null, null];
        }
        let prevSemicolonIndex = null;
        let nextSemicolonIndex = null;
        for (let i = target - 1; i >= 0; i--) {
            if (str[i] === ';') {
                prevSemicolonIndex = i;
                break;
            }
        }
        for (let i = target + 1; i < str.length; i++) {
            if (str[i] === ';') {
                nextSemicolonIndex = i;
                break;
            }
        }
        return [prevSemicolonIndex, nextSemicolonIndex || str.length];
    }
}
exports.LSPServer = LSPServer;
//# sourceMappingURL=lsp-server.js.map