"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParsingWarehouse = void 0;
const antlr4ts_1 = require("antlr4ts");
const atn_1 = require("antlr4ts/atn");
const SparkSqlLexer_1 = require("./lib/SparkSqlLexer");
const SparkSqlParser_1 = require("./lib/SparkSqlParser");
const parse_error_listener_1 = require("./listeners/parse-error.listener");
class ParsingWarehouse {
    constructor() {
        this._warehouse = new Map();
    }
    setParsingValue(document) {
        const parseResult = this.parseDocument(document);
        this._warehouse.set(document.uri, {
            ...parseResult
        });
    }
    parseDocument(document, range) {
        const text = range ? document.getText(range) : document.getText();
        const charStream = antlr4ts_1.CharStreams.fromString(text);
        const lexer = new SparkSqlLexer_1.SparkSqlLexer(charStream);
        const tokenStream = new antlr4ts_1.CommonTokenStream(lexer);
        const parser = new SparkSqlParser_1.SparkSqlParser(tokenStream);
        const errorListener = new parse_error_listener_1.ParseErrorListener(document);
        lexer.removeErrorListeners();
        parser.removeErrorListeners();
        let tree;
        parser.errorHandler = new antlr4ts_1.BailErrorStrategy();
        parser.interpreter.setPredictionMode(atn_1.PredictionMode.SLL);
        try {
            tree = parser.program();
        }
        catch (e) {
            parser.reset();
            parser.errorHandler = new antlr4ts_1.DefaultErrorStrategy();
            parser.addErrorListener(errorListener);
            parser.interpreter.setPredictionMode(atn_1.PredictionMode.LL);
            tree = parser.program();
        }
        return {
            origin: text,
            tokenStream,
            parser,
            tree,
            errors: errorListener.getErrors()
        };
    }
    hasParsingValue(document) {
        return this._warehouse.has(document.uri);
    }
    getParsingValue(document) {
        if (!this._warehouse.has(document.uri)) {
            this.setParsingValue(document);
        }
        return this._warehouse.get(document.uri);
    }
    removeParsingValue(documentUri) {
        this._warehouse.delete(documentUri);
    }
}
exports.ParsingWarehouse = ParsingWarehouse;
//# sourceMappingURL=parsing-warehouse.js.map